/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  display.h  -  Display class
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHLCD_DISPLAY_H
#define GRAPHLCD_DISPLAY_H

#include <string>
#include <vector>
using namespace std;

#include "global.h"
#include "setup.h"
#include "graphlib.h"
#include "logolist.h"
#include "font.h"
#include "widgets.h"
#include "drivers/drivers.h"

#include <vdr/thread.h>
#include <vdr/player.h>


#define LCDMAXCARDS 4
#define MAX_TABS    10

enum ThreadState { Normal, Replay, Menu };

enum eReplayMode { eReplayNormal, eReplayMusic, eReplayDVD, eReplayFile, eReplayImage, eReplayAudioCD };

// Display update Thread
class cGraphLCDDisplay : public cThread
{
public:
  cGraphLCDDisplay();
  ~cGraphLCDDisplay();

  int Init(const char * CfgDir);

  void SetChannel(int ChannelNumber);
  void SetChannelText(const char * Text);
  void SetProgramme(time_t PresentTime, const char * PresentTitle, const char * PresentSubtitle,
                    time_t FollowingTime, const char * FollowingTitle, const char * FollowingSubtitle);
  void SetClear();
  void SetOsdTitle(const char * Title);
  void SetOsdItem(const char * Text, int Index);
  void SetOsdCurrentItem(const char * Text);
  void Recording(const cDevice  * Device , const char * Name);
  void Replaying(const cControl * Control, const char * Name);
  void SetStatusMessage(const char * Msg);
  void SetOsdTextItem(const char * Text, bool Scroll);
  void SetColorButtons(const char * Red, const char * Green, const char * Yellow, const char * Blue);
  void SetVolume(int Volume, bool Absolute);

  void Update();

protected:
  virtual void Action();

private:
  bool update;
  bool active;

  cGraphLCDDevice * pLCDDevice;
  cGraphLCDGraphLib * pGraLib;
  cGraphLCDFont * largeFont;
  cGraphLCDFont * normalFont;
  cGraphLCDFont * smallFont;
  cGraphLCDFont * symbols;
  string cfgDir;
  string fontDir;
  string logoDir;

  ThreadState State;
  ThreadState LastState;

  time_t        CurrTime,
                LastTime,
                LastTimeCheckSym,
                LastTimeModSym;
  struct timeval  CurrTimeval,
                  UpdateAt;

  tChannelID currentChannelID;
  string currentChannel;
  string currentChannelTmp;
  int currentChannelNr;

  time_t presentTime;
  time_t followingTime;
  string presentTitle;
  string presentSubtitle;
  string followingTitle;
  string followingSubtitle;

  std::vector<cScroller> scroller;

  cGraphLCDLogoList * logoList;
  cGraphLCDLogo * logo;

  string replayName;
  cControl * replayControl;
  int replayCurrent,
      replayCurrentLast,
      replayTotal,
      replayProgress,
      replayProgressLast;
  eReplayMode replayMode;

  string osdCurrentItem;
  vector <string> osdItems;
  string osdTitle;
  int menuTop, menuCurrent, menuCount;

  string osdMessage;

  string osdTextItemComplete;
  vector <string> osdTextItemLines;
  int osdTextItemTop;
  int osdTextItemVisibleLines;

  string osdColorButton[4];

  int cardIsRecording[LCDMAXCARDS];
  string cardNameRecording[LCDMAXCARDS];
  int           recCounter1, recCounter2;
  int           iAnzTabs;
  int           iTab[MAX_TABS];
  int           iMaxTab[MAX_TABS];

  char          szETSymbols[32];

  int           iVolume;
  int           iLastVolChange;


  void GetProgramme();

  void DisplayChannel();
  void DisplayTime();
  void DisplayLogo();
  void DisplaySymbols();
  void DisplayProgramm();
  void DisplayReplay(int Current, int Total);
  void DisplayMenu();
  void DisplayMessage();
  void DisplayTextItem();
  void DisplayColorButtons();
  void DisplayVolume();

  void UpdateIn(long usec);
  bool CheckAndUpdateSymbols();
  int WrapText(string & text, vector <string> & lines, cGraphLCDFont * font, int maxTextWidth, int maxLines = 100, bool cutTooLong = true);
  
  /** Check if replay index bigger as one hour */
  bool IndexIsGreaterAsOneHour(int Index) const;  
  /** Translate replay index to string with minute and second MM:SS */
  const char *IndexToMS(int Index) const;
  /** Compare Scroller with new Textbuffer*/
  bool IsScrollerTextChanged(const std::vector<cScroller> & scroller, const std::vector <std::string> & lines) const;
  /** Returns true if Logo loaded and activ*/
  bool IsLogoActive() const;
  /** Returns true if Symbols loaded and activ*/
  bool IsSymbolsActive() const;
};

#endif
