/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  drivers.c  -  global driver constants and functions
 *
 *  (c) 2004 Andreas Regel <andreas.regel AT powarman de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#include <string.h>

#include "drivers.h"


tDriver drivers[] =
{
  {"simlcd", 240, 128, kDriverSimLCD},
  {"gu140x32f", 140,  32, kDriverGU140X32F},
  {"gu256x64-372", 256,  64, kDriverGU256X64_372},
  {"gu256x64-3900", 256, 64, kDriverGU256X64_3900},
  {"hd61830", 240, 128, kDriverHD61830},
  {"ks0108", 128,  64, kDriverKS0108},
  {"ks0713", 128,  64, kDriverKS0713},
  {"sed1330", 320, 240, kDriverSED1330},
  {"sed1520", 120,  32, kDriverSED1520},
  {"t6963", 240, 128, kDriverT6963},
  {"framebuffer", 320, 240, kDriverFramebuffer},
  {"image", 240, 128, kDriverImage},
#ifdef INCLUDE_SERDISPLIB
  {"optrex323", 117, 62, kDriverSerDispOptrex323},
  {"pcd8544", 84, 48, kDriverSerDispPCD8544},
  {"nokia7110", 96, 65, kDriverSerDispNOKIA7110},
 #if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,93))
  {"ericssont2x", 101, 33, kDriverSerDispERICSSONT2X},
 #endif /* SERDISP_VERSION */
 #if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,94))
  {"lsU7s1011a",   96,  32, kDriverSerDispLSU7S1011A},
 #endif /* SERDISP_VERSION */
#endif /* INCLUDE_SERDISPLIB */
  {NULL, 0, 0, kDriverUnknown}
};

tDriver * GetAvailableDrivers(int & count)
{
  for (count = 0; drivers[count].alias; count++)
    ;
  return drivers;
}

cGraphLCDDevice * CreateDriver(int driverID, cGraphLCDSetup * setup)
{
  switch(driverID)
  {
    case kDriverSimLCD: return new cGraphLCDDriverSimLCD(setup);
    case kDriverGU140X32F: return new cGraphLCDDriverGU140X32F(setup);
    case kDriverGU256X64_372: return new cGraphLCDDriverGU256X64_372(setup);
    case kDriverGU256X64_3900: return new cGraphLCDDriverGU256X64_3900(setup);
    case kDriverHD61830: return new cGraphLCDDriverHD61830(setup);
    case kDriverKS0108: return new cGraphLCDDriverKS0108(setup);
    case kDriverKS0713: return new cGraphLCDDriverKS0713(setup);
    case kDriverSED1330: return new cGraphLCDDriverSED1330(setup);
    case kDriverSED1520: return new cGraphLCDDriverSED1520(setup);
    case kDriverT6963: return new cGraphLCDDriverT6963(setup);
    case kDriverFramebuffer: return new cGraphLCDDriverFramebuffer(setup);
    case kDriverImage: return new cGraphLCDDriverImage(setup);
#ifdef INCLUDE_SERDISPLIB
    case kDriverSerDispOptrex323: return new cGraphLCDDriverSerDisp(setup, "OPTREX323");
    case kDriverSerDispPCD8544: return new cGraphLCDDriverSerDisp(setup, "PCD8544");
    case kDriverSerDispNOKIA7110: return new cGraphLCDDriverSerDisp(setup, "NOKIA7110");
 #if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,93))
    case kDriverSerDispERICSSONT2X: return new cGraphLCDDriverSerDisp(setup, "ERICSSONT2X");
 #endif /* SERDISP_VERSION */
 #if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,94))
    case kDriverSerDispLSU7S1011A: return new cGraphLCDDriverSerDisp(setup, "LSU7S1011A");
 #endif /* SERDISP_VERSION */
#endif /* INCLUDE_SERDISPLIB */
    case kDriverUnknown:
    default: return NULL;
  }
}

