/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  drivers.h  -  global driver constants and functions
 *
 *  (c) 2004 Andreas Regel <andreas.regel AT powarman de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#ifndef _LIBGRAPHLCD_DRIVERS_H_
#define _LIBGRAPHLCD_DRIVERS_H_

#include "device.h"
#include "simlcd.h"
#include "gu140x32f.h"
#include "gu256x64-372.h"
#include "gu256x64-3900.h"
#include "hd61830.h"
#include "ks0108.h"
#include "ks0713.h"
#include "sed1330.h"
#include "sed1520.h"
#include "t6963.h"
#include "framebuffer.h"
#include "image.h"
#ifdef INCLUDE_SERDISPLIB
#include "serdisp.h"
#endif /* INCLUDE_SERDISPLIB */


enum eDriver
{
	kDriverUnknown = 0,
	kDriverSimLCD = 1,
	kDriverGU140X32F = 2,
	kDriverHD61830 = 3,
	kDriverKS0108 = 4,
	kDriverKS0713 = 5,
	kDriverSED1330 = 6,
	kDriverSED1520 = 7,
	kDriverT6963 = 8,
	kDriverFramebuffer = 9,
	kDriverGU256X64_372	= 10,
	kDriverImage = 11,
	kDriverGU256X64_3900 = 12,
#ifdef INCLUDE_SERDISPLIB
	kDriverSerDispOptrex323 = 100,
	kDriverSerDispPCD8544 = 101,
	kDriverSerDispNOKIA7110 = 102,
 #if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,93))
	kDriverSerDispERICSSONT2X = 103,
 #endif /* SERDISP_VERSION */
 #if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,94))
	kDriverSerDispLSU7S1011A = 104,
 #endif /* SERDISP_VERSION */
#endif /* INCLUDE_SERDISPLIB */
};

struct tDriver
{
  char * alias;
  int defaultSizeX;
  int defaultSizeY;
  eDriver driverID;
};

tDriver * GetAvailableDrivers(int & count);
cGraphLCDDevice * CreateDriver(int driverID, cGraphLCDSetup * setup);

#endif
