/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 * gu140x32f.h - 8-bit driver module for Noritake GU140x32-F7806
 *               VFD displays. The VFD is operated in it's 8 bit-mode to be
 *               connected to a single PC parallel port.
 *
 * (c) 2003 Andreas Brachold <vdr04 AT deltab de>
 *  based on: HD61830 device
 *              (c) 2001-2003 by Carsten Siebholz <c.siebholz AT t-online de>
 *            lcdproc 0.4 driver hd44780-ext8bit
 *              (c) 1999, 1995 Benjamin Tse <blt AT Comports com>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

/**
 * The VFD should be connected to the parallel printer port of the PC.
 * The connections are:
 * (Note: This pin number might differ with other displays)
 *
 * power               VFD
 * GND              -> (1)
 * VCC(+5VDC 500mA) -> (2)
 * printer port
 *
 *                 NC  (3)
 * nSTRB (1)       RS  (4)
 * nLF  (14)       RW  (5)
 * INIT (16)       EN  (6)
 * D0    (2)       D0  (7)
 * D1    (3)       D1  (8)
 * D2    (4)       D2  (9)
 * D3    (5)       D3 (10)
 * D4    (6)       D4 (11)
 * D5    (7)       D5 (12)
 * D6    (8)       D6 (13)
 * D7    (9)       D7 (14)
 *
 **/

#ifndef _LIBGRAPHLCD_GU140X32F_H_
#define _LIBGRAPHLCD_GU140X32F_H_

#include "device.h"
#include "port.h"
#include "../setup.h"

class cGraphLCDDriverGU140X32F : public cGraphLCDDevice
{
  byte   m_WiringRS;
  byte   m_WiringEN1;
  //byte   m_WiringRW;
  //byte   m_WiringBL;

  cParallelPort * port;

  cGraphLCDSetup * setup;
  cGraphLCDSetup * oldSetup;

  int m_iSizeYb;
  int m_nRefreshCounter;
  byte *m_pDrawMem;             /* the draw "memory" */
  byte *m_pVFDMem;              /* the double buffed display "memory" */
  long m_nTimingAdjustCmd;
  bool m_bSleepIsInit;

protected:
  void ClearVFDMem();
  void Write(byte nFlags, byte bData, unsigned int nMicroSecBusyTime);
  void CheckSetup();

public:
  cGraphLCDDriverGU140X32F(cGraphLCDSetup * setup);
  virtual ~cGraphLCDDriverGU140X32F();

  virtual int Init();
  virtual int DeInit();

  virtual void Clear();
  virtual void SetPixel(int x, int y, eColor color);
  virtual void Set8Pixels(int x, int y, byte data, eColor color);
  virtual void Refresh(bool refreshAll = false);

  virtual void SetBrightness(unsigned int percent);
};

#endif
