/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 * gu256x64-372.h - 8-bit driver module for Noritake GU256x64-372
 *               VFD displays. The VFD is operated in it's 8 bit-mode to be
 *               connected to a single PC parallel port.
 *
 * (c) 20040410 andreas 'randy' weinberger (randy AT smue org)
 *
 *  based on: 
 *            gu256x32f driver module for graphlcd
 *              (c) 2003 Andreas Brachold <vdr04 AT deltab de>
 *            HD61830 device
 *              (c) 2001-2003 by Carsten Siebholz <c.siebholz AT t-online de>
 *            lcdproc 0.4 driver hd44780-ext8bit
 *              (c) 1999, 1995 Benjamin Tse <blt AT Comports com>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

/**
 * The VFD should be connected to the parallel printer port of the PC.
 * The connections are:
 * (Note: This pin number might differ with other displays)
 *
 *   printerport	LCD          other
 *  -----------	       ----------   ----------
 *   D0    (2)		DB0 (15)			(DataBit 0)
 *   D1    (3)		DB1 (13)			(DataBit 1)
 *   D2    (4)		DB2 (11)			(DataBit 2)
 *   D3    (5)		DB3 (09)			(DataBit 3)
 *   D4    (6)		DB4 (07)			(DataBit 4)
 *   D5    (7)		DB5 (05)			(DataBit 5)
 *   D6    (8)		DB6 (03)			(DataBit 6)
 *   D7    (9)		DB7 (01)			(DataBit 7)
 *   INIT  (16)	nWR (17)			(Write)
 *   nSEL  (17)	R/S (19)			(Register Select, C/D)
 *   GND   (18)	nRD (21)			(Read)
 *   GND   (18)	CSS (23)			(ChipSelect)
 *   GND   (18)        GND (02)                        (Ground)
 *
 **/

#ifndef _LIBGRAPHLCD_GU256X64_372_H_
#define _LIBGRAPHLCD_GU256X64_372_H_

#include "device.h"
#include "port.h"
#include "../setup.h"

class cGraphLCDDriverGU256X64_372 : public cGraphLCDDevice
{
  cParallelPort * port;

  cGraphLCDSetup * setup;
  cGraphLCDSetup * oldSetup;

  int m_iSizeYb;
  int m_nRefreshCounter;
  
  byte ** m_pDrawMem;             /* the draw "memory" */
  byte ** m_pVFDMem;              /* the double buffed display "memory" */

  long m_nTimingAdjustCmd;
  bool m_bSleepIsInit;

protected:
  void ClearVFDMem();
  void Write(byte nFlags, byte bData, unsigned int nMicroSecBusyTime);
  void GU256X64Cmd(byte data);
  void GU256X64Data(byte data);
  void CheckSetup();

public:
  cGraphLCDDriverGU256X64_372(cGraphLCDSetup * setup);
  virtual ~cGraphLCDDriverGU256X64_372();

  virtual int Init();
  virtual int DeInit();

  virtual void Clear();
  virtual void SetPixel(int x, int y, eColor color);
  virtual void Set8Pixels(int x, int y, byte data, eColor color);
  virtual void Refresh(bool refreshAll = false);

  virtual void SetBrightness(unsigned int percent);
};

#endif

