/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  hd61830.h  -  HD61830 device
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

/**
 * The LCD should be connected to the parallel printer port of the PC.
 *
 *
 * The connections are:
 * (Note: This pin number might differ with other displays)
 *                       vv
 *  printerport   LCD          other
 *  -----------   ----------   ----------
 *  GND   (18)    GND   (01)   GND          (Ground)
 *                VDD   (02)   +5V          (V Controller)
 *                V0    (03)   +5V..-15V    (Contrast)
 *  INIT  (16)    RS    (04)                (Register Select)
 *  nLF   (14)    R/W   (05)                (Read 1/Write 0)
 *  nSTRB (01)    EN    (06)                (Enable)
 *  D0    (02)    D0    (07)                (DataBit0)
 *  D1    (03)    D1    (08)                (DataBit1)
 *  D2    (04)    D2    (09)                (DataBit2)
 *  D3    (05)    D3    (10)                (DataBit3)
 *  D4    (06)    D4    (11)                (DataBit4)
 *  D5    (07)    D5    (12)                (DataBit5)
 *  D6    (08)    D6    (13)                (DataBit6)
 *  D7    (09)    D7    (14)                (DataBit7)
 *                nCS   (15)   GND          (Chip Select)
 *                nRES  (16)   +5V          (Reset)
 *                VEE   (17)   -12V         (V LCD)
 *                nDO   (18)                (not connected)
 **/

#ifndef _LIBGRAPHLCD_HD61830_H_
#define _LIBGRAPHLCD_HD61830_H_

#include "device.h"
#include "port.h"
#include "../setup.h"

class cGraphLCDDriverHD61830 : public cGraphLCDDevice
{
private:
  cParallelPort * port;
  
  byte ** newLCD; // wanted state
  byte ** oldLCD; // current state
  cGraphLCDSetup * setup;
  cGraphLCDSetup * oldSetup;
  int refreshCounter;
  long timeForPortCmdInNs;
  bool useSleepInit;

  int InitGraphic();
  void Write(byte cmd, byte data);
  void CheckSetup(void);

public:
  cGraphLCDDriverHD61830(cGraphLCDSetup * setup);
  virtual ~cGraphLCDDriverHD61830();

  virtual int Init();
  virtual int DeInit();

  virtual void Clear();
  virtual void SetPixel(int x, int y, eColor color);
  virtual void Set8Pixels(int x, int y, byte data, eColor color);
  virtual void Refresh(bool refreshAll = false);
};


#endif

