/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  ks0713.h   -  KS0713 device
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

/**
 * The LCD should be connected to the parallel printer port of the PC.
 *
 *
 * The connections are:
 * (Note: This pin number might differ with other displays)
 *                       vv
 *  printerport   LCD          other
 *  -----------   ----------   ----------
 *  GND   (18)    Vss   (01)   GND          (Ground)
 *                Vdd   (02)   +3.3V
 *                MI    (03)   +3.3V        (H : 6800 MPU; L : 8080 MPU)
 *  D7    (09)    D7    (04)                (DataBit7)
 *  D6    (08)    D6    (05)                (DataBit6)
 *  D5    (07)    D5    (06)                (DataBit5)
 *  D4    (06)    D4    (07)                (DataBit4)
 *  D3    (05)    D3    (08)                (DataBit3)
 *  D2    (04)    D2    (09)                (DataBit2)
 *  D1    (03)    D1    (10)                (DataBit1)
 *  D0    (02)    D0    (11)                (DataBit0)
 *  nSTRB (01)    EN    (12)                (Enable)
 *                R/W   (13)   GND          (Read/Write)
 *  nSEL  (17)    RS    (14)                (Register Select)
 *                RST   (15)   (Note 1)     (Reset)
 *                CS1B  (16)   GND          (ChipSelect)
 *                LED+  (17)   (Note 2)     (Backlight)
 *                LED-  (18)   GND          (Backlight GND)
 *
 * Note 1: a simple RC circuit triggers the reset pin at power-on.
 * Note 2: depends on the display. Please check Your data sheet.
 *
 **/

#ifndef _LIBGRAPHLCD_KS0713_H_
#define _LIBGRAPHLCD_KS0713_H_

#include "device.h"
#include "port.h"
#include "../setup.h"


class cGraphLCDDriverKS0713 : public cGraphLCDDevice
{
private:
  cParallelPort * port;
  byte ** LCD;      // linear lcd display "memory"
  byte ** LCD_page; // paged lcd display "memory"
  int refreshCounter;
  long timeForPortCmdInNs;
  cGraphLCDSetup * setup;
  cGraphLCDSetup * oldSetup;
  bool useSleepInit;

  int InitGraphic();
  void Write(byte data, bool C_nD);
  void CheckSetup();

public:
  cGraphLCDDriverKS0713(cGraphLCDSetup * setup);
  virtual ~cGraphLCDDriverKS0713();

  virtual int Init();
  virtual int DeInit();

  virtual void Clear();
  virtual void SetPixel(int x, int y, eColor color);
  virtual void Set8Pixels(int x, int y, byte data, eColor color);
  virtual void Refresh(bool refreshAll = false);
};

#endif

