/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  port.h  -  parallel port class with low level routines
 *
 *  (c) 2004 Andreas Regel <andreas.regel AT powarman de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#ifndef _LIBGRAPHLCD_PORT_H_
#define _LIBGRAPHLCD_PORT_H_

#include "../global.h"

const int kForward = 0;
const int kReverse = 1;

const unsigned char kStrobeHigh = 0x00; // Pin 1
const unsigned char kStrobeLow  = 0x01;
const unsigned char kAutoHigh   = 0x00; // Pin 14
const unsigned char kAutoLow    = 0x02;
const unsigned char kInitHigh   = 0x04; // Pin 16
const unsigned char kInitLow    = 0x00;
const unsigned char kSelectHigh = 0x00; // Pin 17
const unsigned char kSelectLow  = 0x08;

class cParallelPort
{
private:
  int fd;
  int port;
  bool usePPDev;
public:
  cParallelPort();
  ~cParallelPort();

  int Open(int port);
  int Open(char * device);
  int Close();

  bool IsDirectIO() const { return (!usePPDev); }
  int GetPortHandle() const { return ((usePPDev) ? fd : port); }

  void Claim();
  void Release();

  void SetDirection(int direction);
  byte ReadControl();
  void WriteControl(byte values);
  byte ReadStatus();
  byte ReadData();
  void WriteData(byte data);
};

#endif
