/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  sed1520.h  -  SED1520 device
 *
 *  (c) 2003 andreas 'randy' weinberger <vdr AT smue org>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

/**
 * The LCD should be connected to the parallel printer port of the PC.
 *
 *
 * The connections are with Electronic Assembly DIP122 LCD:
 * (Note: This pin number might differ with other displays)
 *                       vv
 *  printerport   LCD          other
 *  -----------   ----------   ----------
 *  GND   (18)    GND   (01)   GND          (Ground)
 *                Vdd   (02)   +5V
 *                Vo    (03)                (LCD Contrast In -4V)
 *  nSEL  (17)    R/S   (04)                (Register Select)
 *                R/W   (05)   GND          (Read/Write)
 *  nSTRB (01)    EN1   (06)                (Enable Left)
 *  D0    (02)    D0    (07)                (DataBit0)
 *  D1    (03)    D1    (08)                (DataBit1)
 *  D2    (04)    D2    (09)                (DataBit2)
 *  D3    (05)    D3    (10)                (DataBit3)
 *  D4    (06)    D4    (11)                (DataBit4)
 *  D5    (07)    D5    (12)                (DataBit5)
 *  D6    (08)    D6    (13)                (DataBit6)
 *  D7    (09)    D7    (14)                (DataBit7)
 *  INIT  (16)    EN2   (15)                (Enable Right)
 *                RESET (16)   +5V          (Controller Reset)
 *                LED+  (17)   +4V/150mA    (Backlight with 6,4Ohm Resistor to +5V)
 *  (nLF) (14)    LED-  (18)   GND          (Backlight GND [via BS170])
 *
 * see also http://www.usblcd.de/lcdproc/sed1520-dip.php4 ...
 *  
 **/

#ifndef _LIBGRAPHLCD_SED1520_H_
#define _LIBGRAPHLCD_SED1520_H_

#include "device.h"
#include "port.h"
#include "../setup.h"

class cGraphLCDDriverSED1520 : public cGraphLCDDevice
{
private:
  cParallelPort * port;
  byte ** LCD;      // linear lcd display "memory"
  byte ** LCD_page; // paged lcd display "memory"
  int refreshCounter;
  long timeForPortCmdInNs;
  cGraphLCDSetup * setup;
  cGraphLCDSetup * oldSetup;
  bool useSleepInit;

  int SEAD;
  int SEPA;
  int SEDS;
  int DION;
  int DIOF;

  int CS1LO;
  int CS2LO;
  int CS1HI;
  int CS2HI;
                    
  int CDHI;
  int CDLO;

  int LED;
  int LEDHI;

  int InitGraphic();
  void SED1520Cmd(byte data, int cmscd);
  void SED1520Data(byte data, int datacs);
  void CheckSetup();

public:
  cGraphLCDDriverSED1520(cGraphLCDSetup * setup);
  virtual ~cGraphLCDDriverSED1520();

  virtual int Init();
  virtual int DeInit();

  virtual void Clear();
  virtual void SetPixel(int x, int y, eColor color);
  virtual void Set8Pixels(int x, int y, byte data, eColor color);
  virtual void Refresh(bool refreshAll = false);
};

#endif

