/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  serdisp.c  -  device supported by serdisplib
 *                http://serdisplib.sourceforge.net
 *
 *  (C) 2003-2004 by Wolfgang Astleitner <mrwastl@users.sourceforge.net>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <vdr/tools.h>

#include "fct.h"

#include "serdisp.h"

#if (SERDISP_VERSION_CODE < SERDISP_VERSION(1,93))
#include "serdisplib/serdisp_parport.h"
#else
#include "serdisplib/serdisp_connect.h"
#endif

#include "serdisplib/serdisp_control.h"


cGraphLCDDriverSerDisp::cGraphLCDDriverSerDisp(cGraphLCDSetup * setup, char * idrivername)
: setup(setup)
{
  int dsizex = 0, dsizey = 0;

//  esyslog("graphlcd plugin: Debug: entering init. drivername %s (cGraphLCDDriver::Init)\n",
//          drivername /* should be replaced through useful err-msgs */);

  oldSetup = new cGraphLCDSetup();
  oldSetup->CopyFrom(setup);

  drivername = idrivername;

//  esyslog("graphlcd plugin: Debug: before cParallelPort. drivername %s (cGraphLCDDriver::Init)\n",
//          drivername /* should be replaced through useful err-msgs */);
  port = new cParallelPort();
//  esyslog("graphlcd plugin: Debug: after cParallelPort. drivername %s (cGraphLCDDriver::Init)\n",
//          drivername /* should be replaced through useful err-msgs */);

  dd = (serdisp_t*)0;

  if (strcasecmp("OPTREX323", drivername) == 0) {
    dsizex = 117; dsizey = 62;
  } else if (strcasecmp("PCD8544", drivername) == 0) {
    dsizex = 84; dsizey = 48;
  } else if (strcasecmp("NOKIA7110", drivername) == 0) {
    dsizex = 96; dsizey = 65;
#if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,93))
  } else if (strcasecmp("ERICSSONT2X", drivername) == 0) {
    dsizex = 101; dsizey = 33;
#endif
#if (SERDISP_VERSION_CODE >= SERDISP_VERSION(1,94))
  } else if (strcasecmp("LSU7S1011A", drivername) == 0) {
    dsizex = 96; dsizey = 32;
#endif
  }

  if (setup->SizeX <= 0) {
    sizeX = dsizex;
  } else {
    sizeX = setup->SizeX;
  }

  if (setup->SizeY <= 0) {
    sizeY = dsizey;
  } else {
    sizeY = setup->SizeY;
  }


//  esyslog("graphlcd plugin: Debug: init done. drivername %s (cGraphLCDDriver::Init)\n",
//          drivername /* should be replaced through useful err-msgs */);
}


cGraphLCDDriverSerDisp::~cGraphLCDDriverSerDisp(void) {
  delete oldSetup;
}


int cGraphLCDDriverSerDisp::Init(void) {
#if (SERDISP_VERSION_CODE < SERDISP_VERSION(1,93))
  serdisp_PP_t* sdcd;
#else
  serdisp_CONN_t* sdcd;
#endif
/*
  if (!setup->PPDevice) {
    // use DirectIO
    if (port->Open(setup->PPPort) != 0)
      return -1;
    uSleep(10);
  } else {
    // use ppdev
    if (port->Open(setup->PPDevice) != 0)
      return -1;
  }
*/

  if (!setup->PPDevice) {
    char * temp = (char*)malloc(sizeof(char)*strlen("0xYYY")+1);
    snprintf(temp, 5, "0x%x", setup->PPPort);
    // use DirectIO
#if (SERDISP_VERSION_CODE < SERDISP_VERSION(1,93))
    if ((sdcd = PP_open(temp)) == 0)
#else
    if ((sdcd = SDCONN_open(temp)) == 0)
#endif
      return -1;
    uSleep(10);
  } else {
    // use ppdev
#if (SERDISP_VERSION_CODE < SERDISP_VERSION(1,93))
    if ((sdcd = PP_open(setup->PPDevice)) == 0)
#else
    if ((sdcd = SDCONN_open(setup->PPDevice)) == 0)
#endif
      return -1;
  }
//  esyslog("graphlcd plugin: Debug: before PP_import. isDirectIO: %d. drivername %s (cGraphLCDDriver::Init)\n",
//          (port->IsDirectIO())?1:0, drivername /* should be replaced through useful err-msgs */);
//  ppd = PP_import((port->IsDirectIO())?1:0, port->GetPortHandle());
//  esyslog("graphlcd plugin: Debug: after PP_open,before serdisp_init. isDirectIO: %d. drivername %s (cGraphLCDDriver::Init)\n",
//          ((port->IsDirectIO())?1:0), drivername /* should be replaced through useful err-msgs */);

  dd = serdisp_init (sdcd, drivername, "");

//  esyslog("graphlcd plugin: Debug: after serdisp_init. handle: %d. drivername %s (cGraphLCDDriver::Init)\n",
//          port->GetPortHandle(), drivername /* should be replaced through useful err-msgs */);

  if (!dd) {
    esyslog("graphlcd plugin: ERROR cannot open display %s. Err:%s (cGraphLCDDriver::Init)\n",
            drivername, "no handle" /* should be replaced through useful err-msgs */);
    return -1;
  }

  serdisp_feature(dd, FEATURE_ROTATE, setup->UpsideDown);
  serdisp_feature(dd, FEATURE_CONTRAST, setup->Contrast);
  serdisp_feature(dd, FEATURE_BACKLIGHT, setup->Backlight);
  serdisp_feature(dd, FEATURE_REVERSE, setup->Invert);

  oldSetup->CopyFrom(setup);

  // clear display
  Clear();
  Refresh(true);

  isyslog("graphlcd plugin: %s initialized.\n", drivername);
  return 0;
}


int cGraphLCDDriverSerDisp::DeInit(void) {
#if (SERDISP_VERSION_CODE < SERDISP_VERSION(1,93))
  serdisp_close(dd);
  PP_close(dd->ppd);
#else
  serdisp_quit(dd);
#endif
  if (port->Close() != 0)
    return -1;
  return 0;
}


void cGraphLCDDriverSerDisp::Clear(void) {
  serdisp_clearbuffer(dd);
}


void cGraphLCDDriverSerDisp::SetPixel(int x, int y, eColor color) {
  serdisp_setpixel(dd, x, y, (color == colorBlack) ? 1 : 0);
}


void cGraphLCDDriverSerDisp::Set8Pixels(int x, int y, byte data, eColor color) {
  int i, start;

  data = ReverseBits(data);

  start = (x >> 3) << 3;

  for (i = 0; i < 8; i++) {
    if((data & (1 << i)))
      serdisp_setpixel(dd, start+i, y, (color == colorBlack) ? 1 : 0 );
  }
}


void cGraphLCDDriverSerDisp::CheckSetup() {
  bool bContrast = (setup->Contrast != oldSetup->Contrast);
  bool bBacklight = (setup->Backlight != oldSetup->Backlight);
  bool bUpsideDown = (setup->UpsideDown != oldSetup->UpsideDown);
  bool bInvert = (setup->Invert != oldSetup->Invert);

  if (setup->DiffersImportantlyWith(oldSetup)) {
    DeInit();
    Init();
  } else if (bContrast) {
    serdisp_feature(dd, FEATURE_CONTRAST, setup->Contrast);
    oldSetup->Contrast = setup->Contrast;
  } else if (bBacklight) {
    serdisp_feature(dd, FEATURE_BACKLIGHT, setup->Backlight);
    oldSetup->Backlight = setup->Backlight;
  } else if (bUpsideDown) {
    serdisp_feature(dd, FEATURE_ROTATE, setup->UpsideDown);
    oldSetup->UpsideDown = setup->UpsideDown;
  } else if (bInvert) {
    serdisp_feature(dd, FEATURE_REVERSE, setup->Invert);
    oldSetup->Invert = setup->Invert;
  }
}


void cGraphLCDDriverSerDisp::Refresh(bool bRefreshAll) {
  CheckSetup();

  if (bRefreshAll)
    serdisp_rewrite(dd);
  else
    serdisp_update(dd);
}

