/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  simlcd.c   -  SimLCD device
 *                Output goes to a file instead of lcd.
 *                Use SimLCD tool to view this file
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <vdr/tools.h>

#include "simlcd.h"
#include "fct.h"



cGraphLCDDriverSimLCD::cGraphLCDDriverSimLCD(cGraphLCDSetup * setup)
: setup(setup)
{
  int x;

  oldSetup = new cGraphLCDSetup();
  oldSetup->CopyFrom(setup);

  sizeX = setup->SizeX;
  if (sizeX < 0)
    sizeX = 0;
  sizeY = setup->SizeY;
  if (sizeY < 0)
    sizeY = 0;

  // setup lcd array
  LCD = new byte*[(sizeX + 7) / 8];
  if (LCD)
  {
    for (x = 0; x < (sizeX + 7) / 8; x++)
    {
      LCD[x] = new byte[sizeY];
      memset(LCD[x], 0, sizeY);
    }
  }
}

cGraphLCDDriverSimLCD::~cGraphLCDDriverSimLCD()
{
  int x;

  // free lcd array
  if (LCD)
  {
    for (x = 0; x < (sizeX + 7) / 8; x++)
    {
      delete[] LCD[x];
    }
    delete[] LCD;
  }
  delete oldSetup;
}


int cGraphLCDDriverSimLCD::Init()
{
  oldSetup->CopyFrom(setup);

  // clear display
  Clear();
  Refresh(true);

  isyslog("graphlcd plugin: SIMLCD initialized.\n");
  return 0;
}


int cGraphLCDDriverSimLCD::DeInit()
{
  return 0;
}


void cGraphLCDDriverSimLCD::CheckSetup()
{
  if (setup->DiffersImportantlyWith(oldSetup))
  {
    DeInit();
    Init();
  }
}


void cGraphLCDDriverSimLCD::Clear()
{
  for (int x = 0; x < (sizeX + 7) / 8; x++)
    memset(LCD[x], 0, sizeY);
}


void cGraphLCDDriverSimLCD::SetPixel(int x, int y, eColor color)
{
  byte c;

  clip(x, 0, sizeX - 1);
  clip(y, 0, sizeY - 1);

  c = 0x80;
  if (setup->UpsideDown)
  {
    x = sizeX - 1 - x;
    y = sizeY - 1 - y;
  }

  c = c >> (x % 8);
  if (color == colorBlack)
  {
    LCD[x / 8][y] = LCD[x / 8][y] | c;
  }
  else
  {
    LCD[x / 8][y] = LCD[x / 8][y] & ~c;
  }
}


void cGraphLCDDriverSimLCD::Set8Pixels(int x, int y, byte data, eColor color)
{
  clip(x, 0, sizeX - 1);
  clip(y, 0, sizeY - 1);

  if (!setup->UpsideDown)
  {
    // normal orientation
    if (color == colorBlack)
    {
      LCD[x / 8][y] = LCD[x / 8][y] | data;
    }
    else
    {
      LCD[x / 8][y] = LCD[x / 8][y] & ~data;
    }
  }
  else
  {
    // upside down orientation
    x = sizeX - 1 - x;
    y = sizeY - 1 - y;
    if (color == colorBlack)
    {
      LCD[x / 8][y] = LCD[x / 8][y] | ReverseBits(data);
    }
    else
    {
      LCD[x / 8][y] = LCD[x / 8][y] & ReverseBits(~data);
    }
  }
}


void cGraphLCDDriverSimLCD::Refresh(bool refreshAll)
{
  FILE * fp = NULL;
  int x;
  int y;
  int i;
  unsigned char c;

  CheckSetup();

  fp = fopen("/tmp/simlcd.sem", "r");
  if (!fp || refreshAll)
  {
    if(fp)
      fclose(fp);
    fp = fopen("/tmp/simlcd.dat", "w");
    if (fp)
    {
      for (y = 0; y < sizeY; y++)
      {
        for (x = 0; x < (sizeX + 7) / 8; x++)
        {
          c = LCD[x][y] ^ ((setup->Invert != 0) ? 0xff : 0x00);
          for (i = 0; i < 8; i++)
          {
            if (c & 0x80)
            {
              fprintf(fp,"#");
            }
            else
            {
              fprintf(fp,".");
            }
            c = c << 1;
          }
        }
        fprintf(fp,"\n");
      }
      fclose(fp);
    }

    fp = fopen("/tmp/simlcd.sem", "w");
    fclose(fp);
  }
  else
  {
    fclose(fp);
  }
}
