/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  t6963.h  -  T6963 device
 *
 *  (c) 2003, 2004 Andreas Regel <andreas.regel AT gmx de>
 *   based on: HD61830 device
 *              (c) 2001-2004 Carsten Siebholz
 *             lcdproc 0.5 driver
 *              (c) 2001 Manuel Stahl
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

/**
 * The LCD should be connected to the parallel printer port of the PC.
 *
 *
 * The connections are:
 * (Note: This pin number might differ with other displays)
 *                       vv
 *  printerport   LCD          other
 *  -----------   ----------   ----------
 *  GND   (18)    GND   (02)   GND          (Ground)
 *  nSTRB (01)    WR    (05)                (Write)
 *  nSEL  (17)    RD    (06)                (Read)
 *  nLF   (14)    CE    (07)                ()
 *  INIT  (16)    C/D   (08)                ()
 *  D0    (02)    D0    (11)                (DataBit0)
 *  D1    (03)    D1    (12)                (DataBit1)
 *  D2    (04)    D2    (13)                (DataBit2)
 *  D3    (05)    D3    (14)                (DataBit3)
 *  D4    (06)    D4    (15)                (DataBit4)
 *  D5    (07)    D5    (16)                (DataBit5)
 *  D6    (08)    D6    (17)                (DataBit6)
 *  D7    (09)    D7    (18)                (DataBit7)
 *
 *
 *
 * Alternative wiring  (used by some Windows programs):
 *
 *  printerport   LCD          other
 *  -----------   ----------   ----------
 *  GND   (18)    GND   (02)   GND          (Ground)
 *  INIT  (16)    WR    (05)                (Write)
 *  nLF   (14)    RD    (06)                (Read)
 *  nSTRB (01)    CE    (07)                ()
 *  nSEL  (17)    C/D   (08)                ()
 *  D0    (02)    D0    (11)                (DataBit0)
 *  D1    (03)    D1    (12)                (DataBit1)
 *  D2    (04)    D2    (13)                (DataBit2)
 *  D3    (05)    D3    (14)                (DataBit3)
 *  D4    (06)    D4    (15)                (DataBit4)
 *  D5    (07)    D5    (16)                (DataBit5)
 *  D6    (08)    D6    (17)                (DataBit6)
 *  D7    (09)    D7    (18)                (DataBit7)
 *
 *
 * Note: the LCD must operate in the 6x8 or 8x8 Font mode.
 *   Usually, there is a pin to choose 6x8 or 8x8 Font mode.
 *
 *   There are Displays that have a pin to select the number of
 *   characters per line. In that case, You have to select
 *   (horiz.Size / 6) or (horiz.Size / 8) characters per line.
 **/

#ifndef _LIBGRAPHLCD_T6963_H_
#define _LIBGRAPHLCD_T6963_H_

#include "device.h"
#include "port.h"
#include "../setup.h"

class cGraphLCDDriverT6963 : public cGraphLCDDevice
{
private:
  cParallelPort * port;
  byte ** newLCD; // wanted state
  byte ** oldLCD; // current state
  cGraphLCDSetup * setup;
  cGraphLCDSetup * oldSetup;
  int refreshCounter;
  int bidirectLPT;
  int displayMode;

  int FS;
  int WRHI;
  int WRLO;
  int RDHI;
  int RDLO;
  int CEHI;
  int CELO;
  int CDHI;
  int CDLO;
  bool autoWrite;

  void T6963SetControl(byte flags);
  void T6963DSPReady();
  void T6963Data(byte data);
  void T6963Command(byte cmd);
  void T6963CommandByte(byte cmd, byte data);
  void T6963Command2Bytes(byte cmd, byte data1, byte data2);
  void T6963CommandWord(byte cmd, word data);
  void T6963DisplayMode(byte mode, bool enable);

  void CheckSetup();

public:
  cGraphLCDDriverT6963(cGraphLCDSetup * setup);
  virtual ~cGraphLCDDriverT6963();

  virtual int Init();
  virtual int DeInit();

  virtual void Clear();
  virtual void SetPixel(int x, int y, eColor color);
  virtual void Set8Pixels(int x, int y, byte data, eColor color);
  virtual void Refresh(bool refreshAll = false);
};

#endif
