/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  logolist.c  -  logo list class
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>

#include "logolist.h"
#include "strfct.h"

#include <vdr/tools.h>


#define ALIAS_FILE "logonames.alias"


cGraphLCDLogoList::cGraphLCDLogoList(const char * logodir, const char* cfgdir)
{
  FILE * aliasFile;
  static char line[256];
  tAliasListElement * newAlias;
  char * aliasFileName;
  char * tmpstr;

  logoDir = strdup(logodir);
  aliasFileName = strdup(AddDirectory(cfgdir, ALIAS_FILE));

  aliasFile = fopen(aliasFileName, "r");
  if (aliasFile)
  {
    while (!feof(aliasFile))
    {
      strcpy(line, "");
      fgets(line, sizeof(line), aliasFile);
      trim(line);
      if (line[0] != '#')
      {
        tmpstr = strchr(line, ':');
        if (tmpstr)
        {
          if (*(tmpstr + 1))
          {
            *tmpstr = 0;

            newAlias = new tAliasListElement;
            if (newAlias)
            {
              newAlias->channel = strdup(line);
              newAlias->alias = strdup(tmpstr + 1);

              aliasList.push_back(newAlias);
            }
          }
        }
      }
    }
    fclose(aliasFile);
  }
  free (aliasFileName);

//  list <tAliasListElement *>::iterator it;
//  for (it = aliasList.begin(); it != aliasList.end(); it++)
//  {
//    newAlias = *(it.operator->());
//    printf(">>>>>> AliasList:  >%s< : >%s<\n", newAlias->channel, newAlias->alias);
//  }
}


cGraphLCDLogoList::~cGraphLCDLogoList()
{
  tAliasListElement * tmpAlias;
  list <tAliasListElement *>::iterator it;

  for (it = aliasList.begin(); it != aliasList.end(); it++)
  {
    tmpAlias = *(it.operator->());
    free (tmpAlias->channel);
    free (tmpAlias->alias);
    delete[] tmpAlias;
  }
  aliasList.empty();

  for (unsigned int i = 0; i < logos.size(); i++)
    delete logos[i].logo;

  free (logoDir);
}


char * cGraphLCDLogoList::CreateFullFileName(const char * chName, ePicType type)
{
  char *retstr = NULL;
  char *tmpstr;

  tmpstr = strdup(AddDirectory(logoDir, chName));

  retstr = new char[strlen(tmpstr) + 2 + strlen(GLCDFILE_EXT) + 1];
  strcpy(retstr, tmpstr);
  free (tmpstr);

  switch (type)
  {
    case PIC_FIXED:  // do not attach anything
      break;
    case LOGO_SMALL:
      strcat(retstr, "_s");
      break;
    case LOGO_MEDIUM:
      strcat(retstr, "_m");
      break;
    case LOGO_LARGE:
      strcat(retstr, "_l");
      break;
  }
  strcat(retstr, GLCDFILE_EXT);

  return retstr;
}


char * cGraphLCDLogoList::CheckAndGetAlias(const char* chName)
{
  char * retstr = NULL;
  tAliasListElement * tmpAlias;
  list <tAliasListElement *>::iterator it;
  unsigned int len;

  len = strlen(chName);

  for (it = aliasList.begin(); it != aliasList.end(); it++)
  {
    tmpAlias = *(it.operator->());
    if (!strncmp(tmpAlias->channel, chName, len))
    {
      retstr = tmpAlias->alias;
      break;
    }
  }

  return retstr;
}


cGraphLCDLogo * cGraphLCDLogoList::GetLogo(int channelNr, const char * chName, ePicType type)
{
  unsigned int i;

  char * logoFileName = NULL;
  char * tmpstr;
  FILE * inFile;

  for (i = 0; i < logos.size(); i++)
  {
    if (logos[i].channelNr == channelNr)
    {
      return logos[i].logo;
    }
  }

  // not found in cache => load logo
  // load new logo
  // check the name first
  logoFileName = CreateFullFileName(chName, type);
  inFile = fopen(logoFileName, "rb");
  if (!inFile)
  {
    tmpstr = CheckAndGetAlias(chName);
    if (tmpstr)
    {
      delete[] logoFileName;
      logoFileName = CreateFullFileName(tmpstr, type);
      inFile = fopen(logoFileName, "rb");
    }
  }

  if (inFile)
  {
    fclose(inFile);

    // try to load logo
    tLogoListElement newLogo;
    newLogo.channelNr = channelNr;
    newLogo.logo = new cGraphLCDLogo(logoFileName);

    if (newLogo.logo->DataPtr())
    {
      logos.push_back(newLogo);
      return logos[logos.size() - 1].logo;
    }
    else
    {
      delete newLogo.logo;
      return NULL;
    }
  }
  return NULL;
}
