/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  menu.c  -  setup menu class
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#include "menu.h"
#include "drivers/drivers.h"


cGraphLCDMenuSetup::cGraphLCDMenuSetup()
{
  static const char * wiringValuesSED1330[4];
  wiringValuesSED1330[0]=tr("Original");
  wiringValuesSED1330[1]=tr("PowerLCD");
  wiringValuesSED1330[2]=tr("LCDProc");
  wiringValuesSED1330[3]=tr("Tweakers.net");
  wiringValuesSED1330[4]=tr("YASEDW");
  static const char * interfaceValues[2];
  interfaceValues[0]=tr("6800");
  interfaceValues[1]=tr("8080");
  static const char * wiringValuesT6963[3];
  wiringValuesT6963[0]=tr("Standard");
  wiringValuesT6963[1]=tr("Windows");
  wiringValuesT6963[2]=tr("Feegy");
  static const char * fontSizeValues[2];
  fontSizeValues[0]=tr("6x8");
  fontSizeValues[1]=tr("8x8");
  static const char * wiringValuesGU256X64[6];
  wiringValuesGU256X64[0]=tr("Parallel DMA");
  wiringValuesGU256X64[1]=tr("Parallel normal");
  wiringValuesGU256X64[2]=tr("Serial");
  wiringValuesGU256X64[3]=tr("none");
  wiringValuesGU256X64[4]=tr("Satyr DMA");
  wiringValuesGU256X64[5]=tr("Satyr normal");
  static const char * waitMethodValues[4];
  waitMethodValues[0]=tr("usleep");
  waitMethodValues[1]=tr("nanosleep");
  waitMethodValues[2]=tr("nanosleep (sched_rr)");
  waitMethodValues[3]=tr("gettimeofday");
  static const char * showDateTimeValues[3];
  showDateTimeValues[0]=tr("no");
  showDateTimeValues[1]=tr("yes");
  showDateTimeValues[2]=tr("not in menu");
  static const char * showSymbolsValues[3];
  showSymbolsValues[0]=tr("no");
  showSymbolsValues[1]=tr("yes");
  showSymbolsValues[2]=tr("compressed");
  static const char * showLogoValues[4];
  showLogoValues[0]=tr("no");
  showLogoValues[1]=tr("auto");
  showLogoValues[2]=tr("medium");
  showLogoValues[3]=tr("large");
  static const char * scrollModeValues[3];
  scrollModeValues[0] = tr("never");
  scrollModeValues[1] = tr("once");
  scrollModeValues[2] = tr("always");

  newGraphLCDSetup = GraphLCDSetup;

  Add(new cMenuEditBoolItem( tr("Plugin active"),           &newGraphLCDSetup.PluginActive, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Orientation"),             &newGraphLCDSetup.UpsideDown, tr("normal"), tr("upside down")));
  Add(new cMenuEditBoolItem( tr("Display"),                 &newGraphLCDSetup.Invert, tr("Black/White"), tr("White/Black")));

  if (newGraphLCDSetup.DriverID == kDriverGU140X32F ||
      newGraphLCDSetup.DriverID == kDriverGU256X64_3900 ||
      newGraphLCDSetup.DriverID == kDriverGU256X64_372)
  {
    Add(new cMenuEditIntItem ( tr("Brightness"),            &newGraphLCDSetup.Brightness, 0,100));
  }	  
#ifdef INCLUDE_SERDISPLIB
  // 2004-03-06 wastl: extra settings for displays supported by serdisplib
  if (newGraphLCDSetup.DriverID == kDriverSerDispOptrex323 || 
      newGraphLCDSetup.DriverID == kDriverSerDispPCD8544 || 
      newGraphLCDSetup.DriverID == kDriverSerDispNOKIA7110)
  {
    Add(new cMenuEditIntItem(tr("Contrast"), &newGraphLCDSetup.Contrast, 0,10));
    Add(new cMenuEditBoolItem(tr("Backlight"), &newGraphLCDSetup.Backlight, tr("no"), tr("yes")));
  }    
#endif /* INCLUDE_SERDISPLIB */
  Add(new cMenuEditIntItem(tr("Refresh Display"), &newGraphLCDSetup.RefreshDisplay, 0,50));
  Add(new cMenuEditIntItem(tr("Adjust Timing"), &newGraphLCDSetup.AdjustTiming, -50,50));
  Add(new cMenuEditStraItem(tr("Wait Method"), &newGraphLCDSetup.WaitMethod, 4, waitMethodValues));
  Add(new cMenuEditIntItem(tr("Wait Priority"), &newGraphLCDSetup.WaitPrio, -20,19));

  if (newGraphLCDSetup.DriverID == kDriverGU140X32F)
  {
    Add(new cMenuEditStraItem(tr("Wiring (GU140X32F)"), &newGraphLCDSetup.Wiring, 2, wiringValuesT6963));
  }

  if (newGraphLCDSetup.DriverID == kDriverGU256X64_3900)
  {
    Add(new cMenuEditStraItem( tr("Wiring (GU256X64_3900)"), &newGraphLCDSetup.Wiring, 6, wiringValuesGU256X64));
  }

  if (newGraphLCDSetup.DriverID == kDriverT6963)
  {
    Add(new cMenuEditStraItem(tr("Wiring (T6963)"), &newGraphLCDSetup.Wiring, 3, wiringValuesT6963));
    Add(new cMenuEditStraItem(tr("Font Select (T6963)"), &newGraphLCDSetup.FontSize, 2, fontSizeValues));
    Add(new cMenuEditBoolItem(tr("Auto Mode (T6963)"), &newGraphLCDSetup.AutoMode, tr("no"), tr("yes")));
    Add(new cMenuEditBoolItem(tr("Status Check (T6963)"), &newGraphLCDSetup.StatusCheck, tr("no"), tr("yes")));
  }
  
  if (newGraphLCDSetup.DriverID == kDriverSED1330)
  {
    Add(new cMenuEditIntItem(tr("Oscillator Frequency (kHz)"), &newGraphLCDSetup.OscillatorFrequency, 1000, 15000));
    Add(new cMenuEditStraItem(tr("Wiring"), &newGraphLCDSetup.Wiring, 5, wiringValuesSED1330));
    Add(new cMenuEditStraItem(tr("Interface family"), &newGraphLCDSetup.Interface, 2, interfaceValues));
  }

  Add(new cMenuEditStraItem( tr("Show Date/Time"),          &newGraphLCDSetup.ShowDateTime, 3, showDateTimeValues));
  Add(new cMenuEditBoolItem( tr("Show Channel"),            &newGraphLCDSetup.ShowChannel, tr("no"), tr("yes")));
  Add(new cMenuEditStraItem( tr("Show Logo"),               &newGraphLCDSetup.ShowLogo, 4, showLogoValues));
  Add(new cMenuEditStraItem( tr("Show Symbols"),            &newGraphLCDSetup.ShowSymbols, 3, showSymbolsValues));
  Add(new cMenuEditBoolItem( tr("Show ET Symbols"),         &newGraphLCDSetup.ShowETSymbols, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Show Program"),            &newGraphLCDSetup.ShowProgram, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Show Timebar"),            &newGraphLCDSetup.ShowTimebar, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Show Menu"),               &newGraphLCDSetup.ShowMenu, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Show Messages"),           &newGraphLCDSetup.ShowMessages, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Show Color Buttons"),      &newGraphLCDSetup.ShowColorButtons, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Show Volume"),             &newGraphLCDSetup.ShowVolume, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Show free Cards"),         &newGraphLCDSetup.ShowNotRecording, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Identify replay type"),    &newGraphLCDSetup.IdentifyReplayType, tr("no"), tr("yes")));
  if (newGraphLCDSetup.IdentifyReplayType)
  {
  	Add(new cMenuEditBoolItem( tr("Modify replay string"), &newGraphLCDSetup.ModifyReplayString, tr("no"), tr("yes")));
    Add(new cMenuEditStraItem( tr("Show Logo on Replay"), &newGraphLCDSetup.ReplayLogo, 4, showLogoValues));
  }
  Add(new cMenuEditStraItem( tr("Scroll text lines"),       &newGraphLCDSetup.ScrollMode, 3, scrollModeValues));
  Add(new cMenuEditIntItem( tr("Scroll speed"),          &newGraphLCDSetup.ScrollSpeed, 1, 10));
  Add(new cMenuEditIntItem( tr("Scroll time interval"),  &newGraphLCDSetup.ScrollTime, 100, 2000));
}


void cGraphLCDMenuSetup::Store()
{
  SetupStore("Plugin active", GraphLCDSetup.PluginActive  = newGraphLCDSetup.PluginActive);
  SetupStore("Orientation", GraphLCDSetup.UpsideDown      = newGraphLCDSetup.UpsideDown);
  SetupStore("Display",     GraphLCDSetup.Invert          = newGraphLCDSetup.Invert);

  if (newGraphLCDSetup.DriverID == kDriverGU140X32F ||
      newGraphLCDSetup.DriverID == kDriverGU256X64_372)
  {
    SetupStore("Brightness",  GraphLCDSetup.Brightness      = newGraphLCDSetup.Brightness);
  }
  if (newGraphLCDSetup.DriverID == kDriverGU256X64_3900)
  {
    SetupStore("Brightness",  GraphLCDSetup.Brightness      = newGraphLCDSetup.Brightness);
    SetupStore("Wiring",      GraphLCDSetup.Wiring          = newGraphLCDSetup.Wiring);
  }
#ifdef INCLUDE_SERDISPLIB
  // 2004-03-06 wastl: extra settings for displays supported by serdisplib
  if (newGraphLCDSetup.DriverID == kDriverSerDispOptrex323 || 
      newGraphLCDSetup.DriverID == kDriverSerDispPCD8544 || 
      newGraphLCDSetup.DriverID == kDriverSerDispNOKIA7110)
  {
    SetupStore("Contrast",  GraphLCDSetup.Contrast          = newGraphLCDSetup.Contrast);
    SetupStore("Backlight",  GraphLCDSetup.Backlight        = newGraphLCDSetup.Backlight);
  }    
#endif /* INCLUDE_SERDISPLIB */
  SetupStore("RefreshDisplay",GraphLCDSetup.RefreshDisplay= newGraphLCDSetup.RefreshDisplay);
  SetupStore("AdjustTiming",GraphLCDSetup.AdjustTiming    = newGraphLCDSetup.AdjustTiming);
  SetupStore("WaitMethod",  GraphLCDSetup.WaitMethod      = newGraphLCDSetup.WaitMethod);
  SetupStore("WaitPrio",    GraphLCDSetup.WaitPrio        = newGraphLCDSetup.WaitPrio);

  if (GraphLCDSetup.DriverID == kDriverGU140X32F)
  {
    SetupStore("Wiring", GraphLCDSetup.Wiring = newGraphLCDSetup.Wiring);
  }

  if (GraphLCDSetup.DriverID == kDriverT6963)
  {
    SetupStore("Wiring",      GraphLCDSetup.Wiring          = newGraphLCDSetup.Wiring);
    SetupStore("FontSize",    GraphLCDSetup.FontSize        = newGraphLCDSetup.FontSize);
    SetupStore("AutoMode",    GraphLCDSetup.AutoMode        = newGraphLCDSetup.AutoMode);
    SetupStore("StatusCheck", GraphLCDSetup.StatusCheck     = newGraphLCDSetup.StatusCheck);
  }
  
  if (newGraphLCDSetup.DriverID == kDriverSED1330)
  {
    SetupStore("OscillatorFrequency", GraphLCDSetup.OscillatorFrequency = newGraphLCDSetup.OscillatorFrequency);
    SetupStore("Wiring", GraphLCDSetup.Wiring = newGraphLCDSetup.Wiring);
    SetupStore("Interface", GraphLCDSetup.Interface = newGraphLCDSetup.Interface);
  }

  SetupStore("ShowDateTime",GraphLCDSetup.ShowDateTime    = newGraphLCDSetup.ShowDateTime);
  SetupStore("ShowChannel", GraphLCDSetup.ShowChannel     = newGraphLCDSetup.ShowChannel);
  SetupStore("ShowLogo",    GraphLCDSetup.ShowLogo        = newGraphLCDSetup.ShowLogo);
  SetupStore("ShowSymbols", GraphLCDSetup.ShowSymbols     = newGraphLCDSetup.ShowSymbols);
  SetupStore("ShowETSymbols",GraphLCDSetup.ShowETSymbols  = newGraphLCDSetup.ShowETSymbols);
  SetupStore("ShowProgram", GraphLCDSetup.ShowProgram     = newGraphLCDSetup.ShowProgram);
  SetupStore("ShowTimebar", GraphLCDSetup.ShowTimebar     = newGraphLCDSetup.ShowTimebar);
  SetupStore("ShowMenu",    GraphLCDSetup.ShowMenu        = newGraphLCDSetup.ShowMenu);
  SetupStore("ShowMessages",GraphLCDSetup.ShowMessages    = newGraphLCDSetup.ShowMessages);
  SetupStore("ShowColorButtons",GraphLCDSetup.ShowColorButtons = newGraphLCDSetup.ShowColorButtons);
  SetupStore("ShowVolume",  GraphLCDSetup.ShowVolume      = newGraphLCDSetup.ShowVolume);
  SetupStore("ShowNotRecording", GraphLCDSetup.ShowNotRecording = newGraphLCDSetup.ShowNotRecording);
  SetupStore("IdentifyReplayType", GraphLCDSetup.IdentifyReplayType = newGraphLCDSetup.IdentifyReplayType);
  SetupStore("ReplayLogo", GraphLCDSetup.ReplayLogo = newGraphLCDSetup.ReplayLogo);
  SetupStore("ModifyReplayString", GraphLCDSetup.ModifyReplayString = newGraphLCDSetup.ModifyReplayString);
  SetupStore("ScrollMode", GraphLCDSetup.ScrollMode = newGraphLCDSetup.ScrollMode);
  SetupStore("ScrollSpeed", GraphLCDSetup.ScrollSpeed = newGraphLCDSetup.ScrollSpeed);
  SetupStore("ScrollTime", GraphLCDSetup.ScrollTime = newGraphLCDSetup.ScrollTime);
}
