/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  setup.c  -  Setup
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#include <string.h>

#include "setup.h"
#include "global.h"
#include "drivers/drivers.h"


cGraphLCDSetup GraphLCDSetup;

cGraphLCDSetup::cGraphLCDSetup(void) {
  PluginActive= 1;
  DriverID    = kDriverUnknown;
  PPDevice    = NULL;
  PPPort      = -1;
  SizeX       = 0;
  SizeY       = 0;
  Zoom = 0;
  UpsideDown  = 0;
  Invert      = 0;
  Brightness  = 100;
  AdjustTiming= 0;
  RefreshDisplay=5;
  WaitMethod  = 2;
  WaitPrio    = 0;
  Wiring      = 0;
  FontSize    = 0;
  AutoMode    = 0;
  StatusCheck = 1;
  OscillatorFrequency = 6000;
  Interface = 0;
  ShowDateTime= 1;
  ShowChannel = 1;
  ShowLogo    = 1;
  ShowSymbols = 1;
  ShowETSymbols = 0;
  ShowProgram = 1;
  ShowTimebar = 1;
  ShowMenu    = 1;
  ShowMessages= 1;
  ShowColorButtons = 1;
  ShowVolume  = 1;
  ShowNotRecording  = 0;
  IdentifyReplayType = 1;
  ModifyReplayString = 1;
  ReplayLogo  = 1;
  ScrollMode = 0;
  ScrollSpeed = 2;
  ScrollTime = 500;
#ifdef INCLUDE_SERDISPLIB
  Contrast = 5;
  Backlight = 1;
#endif /* INCLUDE_SERDISPLIB */
}

cGraphLCDSetup::~cGraphLCDSetup(void)
{
  if (this->PPDevice) 
    FREENULL(this->PPDevice);
}

cGraphLCDSetup& cGraphLCDSetup::operator= (const cGraphLCDSetup& setup)
{
  CopyFrom(&setup);
  return *this;
}

void
cGraphLCDSetup::CopyFrom(const cGraphLCDSetup* pSource) {
  if(this->PPDevice) FREENULL(this->PPDevice);

  PluginActive= pSource->PluginActive;
  DriverID    = pSource->DriverID;

  if(pSource->PPDevice) {
    this->PPDevice = strdup(pSource->PPDevice);
  } else {
    this->PPDevice = NULL;
  }
  
  PPPort      = pSource->PPPort;
  SizeX       = pSource->SizeX;
  SizeY       = pSource->SizeY;
  Zoom = pSource->Zoom;
  UpsideDown  = pSource->UpsideDown;
  Invert      = pSource->Invert;
  Brightness  = pSource->Brightness;
  AdjustTiming= pSource->AdjustTiming;
  RefreshDisplay=pSource->RefreshDisplay;
  WaitMethod  = pSource->WaitMethod;
  WaitPrio    = pSource->WaitPrio;
  Wiring      = pSource->Wiring;
  FontSize    = pSource->FontSize;
  AutoMode    = pSource->AutoMode;
  StatusCheck = pSource->StatusCheck;
  OscillatorFrequency = pSource->OscillatorFrequency;
  Interface = pSource->Interface;
  ShowDateTime= pSource->ShowDateTime;
  ShowChannel = pSource->ShowChannel;
  ShowLogo    = pSource->ShowLogo;
  ShowSymbols = pSource->ShowSymbols;
  ShowETSymbols = pSource->ShowETSymbols;
  ShowProgram = pSource->ShowProgram;
  ShowTimebar = pSource->ShowTimebar;
  ShowMenu    = pSource->ShowMenu;
  ShowMessages= pSource->ShowMessages;
  ShowColorButtons = pSource->ShowColorButtons;
  ShowVolume  = pSource->ShowVolume;
  ShowNotRecording  = pSource->ShowNotRecording;
  IdentifyReplayType = pSource->IdentifyReplayType;
  ModifyReplayString = pSource->ModifyReplayString;
  ReplayLogo = pSource->ReplayLogo;
  ScrollMode = pSource->ScrollMode;
  ScrollSpeed = pSource->ScrollSpeed;
  ScrollTime = pSource->ScrollTime;
#ifdef INCLUDE_SERDISPLIB
  Contrast    = pSource->Contrast;
  Backlight   = pSource->Backlight;
#endif /* INCLUDE_SERDISPLIB */
}


bool cGraphLCDSetup::DiffersImportantlyWith(const cGraphLCDSetup* pS2)
{
  bool bRet = false;

  if(!bRet
     && kDriverUnknown != this->DriverID 
     && kDriverUnknown != pS2->DriverID 
     && this->DriverID != pS2->DriverID) {
      bRet = true;
  }
  if(!bRet &&
     this->PPDevice && pS2->PPDevice) {
    if(strcmp(this->PPDevice, pS2->PPDevice))
      bRet = true;
  }

  if(!bRet &&
     (this->PPPort != pS2->PPPort) ||
     (this->SizeX  != pS2->SizeX) ||
     (this->SizeY  != pS2->SizeY))
    bRet = true;

  if(!bRet &&
     (this->Wiring   != pS2->Wiring) ||
     (this->FontSize != pS2->FontSize) ||
     (this->OscillatorFrequency != pS2->OscillatorFrequency) ||
     (this->Interface != pS2->Interface))
    bRet = true;

  if(!bRet &&
     (this->WaitMethod != pS2->WaitMethod) ||
     (this->WaitPrio   != pS2->WaitPrio))
    bRet = true;

  return bRet;
}
