/**
 *  GraphLCD plugin for the Video Disk Recorder 
 *
 *  status.c  -  status monitor class
 *
 *  (c) 2001-2004 Carsten Siebholz <c.siebholz AT t-online de>
 **/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

#include "status.h"


cGraphLCDStatus::cGraphLCDStatus()
: display(NULL)
{
} 

cGraphLCDStatus::~cGraphLCDStatus()
{
  if (display)
    delete display;
}


int cGraphLCDStatus::Init(const char * cfgDir)
{
  display = new cGraphLCDDisplay();
  return display->Init(cfgDir);
}


void cGraphLCDStatus::ChannelSwitch(const cDevice * Device, int ChannelNumber)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::ChannelSwitch  %d %d", Device->CardIndex(), ChannelNumber);
    if (ChannelNumber > 0 && Device->IsPrimaryDevice())
    {
      if (ChannelNumber == cDevice::CurrentChannel())
        display->SetChannel(ChannelNumber);
    }
  }
}


void cGraphLCDStatus::Recording(const cDevice * Device, const char * Name)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::Recording  %d %s", Device->CardIndex(), Name);
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::Recording  %d %s\n", Device->CardIndex(), Name);
    display->Recording(Device, Name);
  }
}


void cGraphLCDStatus::Replaying(const cControl * Control, const char * Name)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::Replaying  %s", Name);
    display->Replaying(Control, Name);
  }
}


void cGraphLCDStatus::SetVolume(int Volume, bool Absolute)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::SetVolume  %d %d", Volume, Absolute);
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::SetVolume  %d %d\n", Volume, Absolute);
    display->SetVolume(Volume, Absolute);
  }
}


void cGraphLCDStatus::OsdClear()
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdClear");
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::OsdClear\n");
    display->SetClear();
  }
}


void cGraphLCDStatus::OsdTitle(const char * Title)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdTitle '%s'", Title);
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::OsdTitle '%s'\n", Title);
    display->SetOsdTitle(Title);
  }
}


void cGraphLCDStatus::OsdStatusMessage(const char * Message)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdStatusMessage '%s'", Message);
    if (GraphLCDSetup.ShowMessages)
    {
      display->SetStatusMessage(Message);
    }
  }
}


void cGraphLCDStatus::OsdHelpKeys(const char * Red, const char * Green, const char * Yellow, const char * Blue)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdHelpKeys %s - %s - %s - %s", Red, Green, Yellow, Blue);
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::OsdHelpKeys %s - %s - %s - %s\n", Red, Green, Yellow, Blue);
    if (GraphLCDSetup.ShowColorButtons)
    {
      display->SetColorButtons(Red, Green, Yellow, Blue);
    }
  }
}


void cGraphLCDStatus::OsdItem(const char * Text, int Index)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdItem %s, %d", Text, Index);
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::OsdItem %s, %d\n", Text, Index);
    if (GraphLCDSetup.ShowMenu)
    {
      display->SetOsdItem(Text, Index);
    }
  }
}


void cGraphLCDStatus::OsdCurrentItem(const char * Text)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdCurrentItem %s", Text);
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::OsdCurrentItem %s\n", Text);
    if (GraphLCDSetup.ShowMenu)
    {
      display->SetOsdCurrentItem(Text);
    }
  }
}


void cGraphLCDStatus::OsdTextItem(const char * Text, bool Scroll)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdTextItem %s %d", Text, Scroll);
//    printf(">>>>> graphlcd plugin: cGraphLCDStatus::OsdTextItem %s %d\n", Text, Scroll);
    display->SetOsdTextItem(Text, Scroll);
  }
}


void cGraphLCDStatus::OsdChannel(const char * Text)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdChannel >%s<", Text);
    display->SetChannelText(Text);
  }
}


void cGraphLCDStatus::OsdProgramme(time_t PresentTime, const char * PresentTitle, const char * PresentSubtitle, time_t FollowingTime, const char * FollowingTitle, const char * FollowingSubtitle)
{
  if (GraphLCDSetup.PluginActive)
  {
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdProgramme PT  : %s", PresentTitle);
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdProgramme PST : %s", PresentSubtitle);
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdProgramme FT  : %s", FollowingTitle);
//    dsyslog(">>>>> graphlcd plugin: cGraphLCDStatus::OsdProgramme FST : %s", FollowingSubtitle);
    display->SetProgramme(PresentTime, PresentTitle, PresentSubtitle,
                          FollowingTime, FollowingTitle, FollowingSubtitle);
  }
}

