/***************************************************************************
 *  drawtest.c  -  program to test graphic library
 *
 *  (c) 2004 Andreas Regel <andreas.regel AT powarman de>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

// SLOW_IO_BY_JUMPING tells Linux to use jumps instead of a
//   outb(0x80) to slow down the outb(port) a little bit.
#define SLOW_IO_BY_JUMPING
// REALLY_SLOW_IO tells Linux to use the jmp/outb command
//   4 times instead of only 1 time
#undef  REALLY_SLOW_IO


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <dlfcn.h>
#include <unistd.h>

#include "../../global.h"
#include "../../logo.h"
#include "../../setup.h"
#include "../../graphlib.h"
#include "../../drivers/drivers.h"


#define DEFAULT_PPPORT 0x378

int SysLogLevel;


int main(int argc, char * argv[])
{
  cGraphLCDDevice * deviceLCD = NULL;
  cGraphLCDGraphLib * graphLib = NULL;
  SysLogLevel = 3;

  GraphLCDSetup.DriverID = kDriverGU256X64_3900;
  GraphLCDSetup.WaitMethod = 2;
  GraphLCDSetup.Invert = 0;
  GraphLCDSetup.FontSize = 0;
  GraphLCDSetup.AutoMode = 1;
  GraphLCDSetup.PPPort = DEFAULT_PPPORT;

  GraphLCDSetup.SizeX = 256;
  GraphLCDSetup.SizeY = 64;

  deviceLCD = CreateDriver(GraphLCDSetup.DriverID, &GraphLCDSetup);

  if(deviceLCD->Init())
  {
    fprintf(stderr, "ERROR cannot initialize cGraphLCDDevice\n");
    return 2;
  }
  
  graphLib = new cGraphLCDGraphLib(deviceLCD);
  cGraphLCDFont * font = new cGraphLCDFont();
  font->Load("../../graphlcd/fonts/f_l_l.fnt");

  deviceLCD->Clear();

  graphLib->Text(5, 0, 100, "abcde", font, true, colorBlack, colorWhite);

  deviceLCD->Refresh(true);

  deviceLCD->DeInit();

  return 0;
}

