/***************************************************************************
        showpic.c  -  program to show an image on a LCD
                             -------------------
    begin                : Son Nov 01 10:03:08 CEST 2002
    copyright            : (c) 2002-2004 Carsten Siebholz
    email                : c.siebholz AT t-online de

 T6963 driver:
    Andreas 'powARman' Regel <andreas.regel At gmx de>

 KS0108 + SED1520 driver:
    andreas 'randy' weinberger <vdr AT smue org>, 20030611

 SED1330 driver:
    Roland Praml

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program;                                              *
 *   if not, write to the Free Software Foundation, Inc.,                  *
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *
 *                                                                         *
 ***************************************************************************/

// SLOW_IO_BY_JUMPING tells Linux to use jumps instead of a
//   outb(0x80) to slow down the outb(port) a little bit.
#define SLOW_IO_BY_JUMPING
// REALLY_SLOW_IO tells Linux to use the jmp/outb command
//   4 times instead of only 1 time
#undef  REALLY_SLOW_IO


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <dlfcn.h>
//#include <mcheck.h>   // XXX
#include <unistd.h>
#include <signal.h>


#include "../../logo.h"
#include "../../setup.h"
#include "../../drivers/drivers.h"


static const char *prgname = "showpic";
static const char *version = "0.1.0";

/* 2004-04-03 wastl
   because 'byte' is defined in serdisplib-stuff: undef it here */
#undef byte


typedef unsigned char   byte;
typedef unsigned short  word;
typedef unsigned int    dword;

#define MALLOC(type, size)  (type *)malloc(sizeof(type) * (size))
#define FREENULL(p) (free (p), p = NULL)

#define DEFAULTSLEEPMS 100
#define DEFAULT_PPPORT	 0x378

byte bitmask[8]  = {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
byte bitmaskl[8] = {0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe, 0xff};
byte bitmaskr[8] = {0xff, 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01};


int SysLogLevel;
cGraphLCDDevice    *pLCDDevice = NULL;


// prototypes
void Bitmap (int x, int y, class cGraphLCDLogo* pcBmp, eColor color);
void usage(void);


// draws a bitmap (pcBmp) in color (color)
// at the given position (x,y)
void
Bitmap     (int x, int y,
            class cGraphLCDLogo* pcBmp,
            eColor color) {
  byte  cl=0;
  int   xt,yt;
  byte *pC=(byte*)pcBmp->DataPtr();
  word  temp;
  int   h,w;

  x  %= pLCDDevice->SizeX(); if(x<0) x=0;
  y  %= pLCDDevice->SizeY(); if(y<0) y=0;
  w = pcBmp->Width();
  h = pcBmp->Height();

  if(pC) {
    if(!(x%8)) {
      // Bitmap is byte alligned (0,8,16,...)
      for(yt=0; yt<h; yt++) {
        for(xt=0; xt<(w/8); xt++) {
          cl=(byte) *pC;
          pLCDDevice->Set8Pixels (x+(xt*8), y+yt, cl, color);
          pC++;
        }
        if(w%8) {
          cl=(byte) *pC;
          pLCDDevice->Set8Pixels (x+((w/8)*8), y+yt, cl & bitmaskl[w%8-1], color);
          pC++;
        }
      }
    } else {
      // Bitmap is not byte alligned
      for(yt=0; yt<h; yt++) {
        temp=0;
        for(xt=0; xt<(w+(x%8))/8; xt++) {
          cl=(byte) *(pC+yt*((w+7)/8)+xt);
          temp=temp|((word)cl<<(8-(x%8)));
          cl=(temp&0xff00)>>8;
          if(!xt) {
            // first byte
            pLCDDevice->Set8Pixels (x-(x%8)+(xt*8), y+yt, cl & bitmaskr[x%8], color);
          } else {
            // not the first byte
            pLCDDevice->Set8Pixels (x-(x%8)+(xt*8), y+yt, cl, color);
          }
          temp<<=8;
        }
        if((w+(x%8)+7)/8 != (w+(x%8))/8) {
          // print the rest
          cl=(byte) *(pC+(yt+1)*((w+7)/8)-1);
          temp=temp|((word)cl<<(8-(x%8)));
          cl=(temp&0xff00)>>8;
          pLCDDevice->Set8Pixels (x-(x%8)+(((w+(x%8))/8)*8), y+yt, cl & bitmaskl[(w+x)%8-1], color);
        }
      }
    }
  }
}


static volatile bool        stopProgramm     = false;

static void sighandler(int signal) {

	switch (signal) {
	case SIGINT:
	case SIGQUIT:
	case SIGTERM:
  	stopProgramm = true;
	}
}


int main(int argc, char *argv[]) {
  const char     *picFile = NULL;
  cGraphLCDLogo  *pLogo = NULL;
  bool            bEndless  = false;
  bool            bDelay    = false;
  int             optfile = 0;
  int             nSleepMS = DEFAULTSLEEPMS;

  SysLogLevel = 3;

  GraphLCDSetup.WaitMethod = 2;
/*   GraphLCDSetup.PPPort = DEFAULT_PPPORT; */

  static struct option long_options[] = {
    { "contr",    required_argument, NULL, 'c'},
    { "port",     required_argument, NULL, 'p'},
    { "device",   required_argument, NULL, 'd'},
    { "sizex",    required_argument, NULL, 'x'},
    { "sizey",    required_argument, NULL, 'y'},
    { "sleep",    required_argument, NULL, 's'},
    { "endless",        no_argument, NULL, 'e'},
    { "upside_down",    no_argument, NULL, 'u'},
    { "invert",         no_argument, NULL, 'i'},
    { "wiring",   required_argument, NULL, 'w'},
    { "fontsize", required_argument, NULL, 'f'},
    { "automode",       no_argument, NULL, 'a'},
    { "brightness",required_argument, NULL, 'b'},
/* AustrianCoder: Added timing routine selection */
    { "timing",   required_argument, NULL, 't'},
	{ NULL}
  };

  int c, option_index = 0;
  int driverCount;
  tDriver * drivers = GetAvailableDrivers(driverCount);
  int driverNumber = driverCount;
  while((c=getopt_long(argc,argv,"c:p:d:x:y:s:euiw:f:ab:t:",long_options,&option_index))!=-1) {
    switch(c) {
      case 'c':
        for (int i = 0; i < driverCount; i++)
        {
          if (!strcasecmp(optarg, drivers[i].alias))
          {
            GraphLCDSetup.DriverID = drivers[i].driverID;
            driverNumber = i;
            break;
          }
        }
        break;

      case 'p':
        GraphLCDSetup.PPPort = (unsigned int)strtol(optarg, (char**)NULL, 0);
        break;

      case 'd':
        GraphLCDSetup.PPDevice = strdup(optarg);
        break;

      case 'x':
        GraphLCDSetup.SizeX = atoi(optarg);
        break;

      case 'y':
        GraphLCDSetup.SizeY = atoi(optarg);
        break;

      case 'u':
        GraphLCDSetup.UpsideDown = 1;
        break;

      case 'i':
        GraphLCDSetup.Invert = 1;
        break;

      case 'w':
        GraphLCDSetup.Wiring = atoi(optarg);
        break;

      case 'f':
        GraphLCDSetup.FontSize = atoi(optarg);
        break;

      case 'a':
        GraphLCDSetup.AutoMode = 1;
        break;

      case 's':
        nSleepMS = atoi(optarg);
        bDelay = true;
        break;

      case 'e':
        bEndless = true;
        break;

      case 'b':
        GraphLCDSetup.Brightness = atoi(optarg);
        if(GraphLCDSetup.Brightness<0) GraphLCDSetup.Brightness=0;
        if(GraphLCDSetup.Brightness>100) GraphLCDSetup.Brightness=100;
        break;

      case 't':
        GraphLCDSetup.WaitMethod = atoi(optarg);
        break;

      default:
        usage();
        return 1;
    }
  }

  if (kDriverUnknown == GraphLCDSetup.DriverID ||
      driverNumber == driverCount )
  {
    // no driver specified:
    // => choose SIMLCD
    fprintf(stdout, "Unknown Controller.\n"
                    "  Using 'simlcd' instead...\n");
    GraphLCDSetup.DriverID = kDriverSimLCD;
    driverNumber = 0;
  }

  if (!GraphLCDSetup.PPDevice &&
      GraphLCDSetup.PPPort==-1 &&
      GraphLCDSetup.DriverID != kDriverSimLCD)
  {
    // neither PPDevice nor PPPort given:
    // => choose PPPort and set it to default (0x378)
    fprintf(stdout, "Neither IO-Port nor Parport Device has been specified.\n"
                    "  Using IO-Port 0x%X instead...\n", DEFAULT_PPPORT);
    GraphLCDSetup.PPPort = DEFAULT_PPPORT;
  }

  if(GraphLCDSetup.SizeX == 0) GraphLCDSetup.SizeX = drivers[driverNumber].defaultSizeX;
  if(GraphLCDSetup.SizeY == 0) GraphLCDSetup.SizeY = drivers[driverNumber].defaultSizeY;

  if(optind == argc) {
    usage();
    fprintf(stderr, "ERROR: You have to specify the image\n");
    return 1;
  }

  pLCDDevice = CreateDriver(GraphLCDSetup.DriverID, &GraphLCDSetup);

  if(pLCDDevice->Init()) {
    fprintf(stderr, "ERROR cannot initialize cGraphLCDDevice\n");
    delete pLCDDevice;
    return 2;
  }

	signal(SIGINT, sighandler);
	signal(SIGQUIT, sighandler);
	signal(SIGTERM, sighandler);
	signal(SIGHUP, sighandler);
  
  optfile = optind;
  while(optfile < argc && !stopProgramm) {
    picFile = argv[optfile++];
    pLogo = new cGraphLCDLogo(picFile);
    if(!pLogo) {
      fprintf(stderr, "ERROR cannot create cGraphLCDLogo object\n");
      return 3;
    }
    
    if(bDelay)
      pLogo->SetDelay(nSleepMS);

    while(pLogo->DataPtr() && !stopProgramm)
    {
      pLCDDevice->Clear();
      Bitmap(0,0,pLogo, colorBlack);
      pLCDDevice->Refresh(true);

      if(pLogo->Next(0)) // Select next image
      {
        usleep(pLogo->Delay() * 1000);
      }
      else if(bEndless && argc == (optind+1)) // Endless and one and only image
      {
        pLogo->First(0);
        usleep(pLogo->Delay() * 1000);
      }
      else 
        break;
    };

    delete pLogo;
    
    if(optfile<argc || bEndless)
      usleep(nSleepMS * 1000);
    if(optfile>=argc && bEndless)
      optfile = optind;
  }

  pLCDDevice->DeInit();
  delete pLCDDevice;
  return 0;
}


void
usage(void) {
  #define szIndent "                      "
  int iSlen;
  int driverCount;
  tDriver * drivers = GetAvailableDrivers(driverCount);

  fprintf(stdout, "\n");
  fprintf(stdout, "%s v%s\n", prgname, version);
  fprintf(stdout, "%s is a tool to show an image on a LCD.\n", prgname);
  fprintf(stdout, "The image must be in a special format (*.glcd).\n");
  fprintf(stdout, "You can create such images with the convpic tool.\n\n");
  fprintf(stdout, "  Usage: %s [-c CTR] [-p IOP | -d DEV] [-x SZX] [-y SZY] [-s Sleep] [-uie] file [more files]\n\n", prgname);
  fprintf(stdout, "  -c  --contr       sets the controller   (default: %s)\n", drivers[0]);
  fprintf(stdout, "                    currenty available are:\n");

  iSlen=strlen(szIndent);
  fprintf(stdout, "%s", szIndent);
  for (int i = 0; i < driverCount; i++)
  {
    if (iSlen > 70)
    {
      fprintf(stdout, "\n%s", szIndent);
      iSlen=strlen(szIndent);
    }
    fprintf(stdout, "%s  ", drivers[i].alias);
  }
  fprintf(stdout, "\n");
  fprintf(stdout, "  -p  --port        sets the port address (default: 0x%X)\n",DEFAULT_PPPORT);
  fprintf(stdout, "  -d  --device      sets the parport device\n");
  fprintf(stdout, "  -x  --sizex       sets the x-resolution (default: 240)\n");
  fprintf(stdout, "  -y  --sizey       sets the y-resolution (default: 128)\n");
  fprintf(stdout, "  -u  --upside_down rotates the output by 180 degrees (default: no)\n");
  fprintf(stdout, "  -i  --invert      inverts the output    (default: no)\n");
  fprintf(stdout, "  -w  --wiring      wiring (T6963) (0 - standard, 1 - windows, 2 - feegy, default: 0)\n");
  fprintf(stdout, "                    wiring (GU256x64-3900) (0 - standard DMA, 1 - parallel, 2 - serial, 32 - Satyr DMA, default: 0)\n");
  fprintf(stdout, "  -f  --fontsize    font size (T6963) (0 - 6x8, 1 - 8x8, default: 0)\n");
  fprintf(stdout, "  -a  --automode    use auto mode (T6963) (default: no)\n");
  fprintf(stdout, "  -e  --endless     show all images in endless loop   (default: no)\n");
  fprintf(stdout, "  -s  --sleep       set sleeptime between two images [ms]   (default: %d ms)\n",DEFAULTSLEEPMS);
  fprintf(stdout, "  -b  --brightness  set brightness for display if driver support it [%] (default: %d %)\n",GraphLCDSetup.Brightness);
  fprintf(stdout, "  -t  --timing      set timing method (0 - usleep, 1 - nanosleep,\n");
  fprintf(stdout, "                                       2 - nanosleep (sched_rr), 3 - gettimeofday,\n");
  fprintf(stdout, "                                       default: 2)\n");
  fprintf(stdout, "\n" );
  fprintf(stdout, "You can either specify a port or a device. The output with ports is faster,\n");
  fprintf(stdout, "but You need root privileges.\n");
  fprintf(stdout, "\n" );
  fprintf(stdout, "  examples: %s -c ks0108 -x 128 -y 64 vdr-logo.glcd\n", prgname );
  fprintf(stdout, "            %s -c t6963 -d /dev/parport0 -x 160 -y 128 -u -i vdr-logo.glcd\n", prgname );
  fprintf(stdout, "            %s -c sed1520 -p 0x278 -x 128 -y 64 -u -i vdr-logo.glcd\n", prgname );
  fprintf(stdout, "\n" );
}
