#include "setup.h"
#include "widgets.h"

#include <vdr/tools.h>

cScroller::cScroller()
{
  Reset();
}

void cScroller::Reset()
{
  x = 0;
  y = 0;
  xmax = 0;
  font = NULL;
  text = "";
  active = false;
  update = false;
  position = 0;
  increment = 0;
  lastUpdate = 0;
}

bool cScroller::NeedsUpdate()
{
  if (active && 
      time_ms() - lastUpdate > GraphLCDSetup.ScrollTime)
  {
    update = true;
    return true;
  }
  return false;
}

void cScroller::Init(int X, int Y, int Xmax, cGraphLCDFont * Font, const string & Text)
{
  x = X;
  y = Y;
  xmax = Xmax;
  font = Font;
  text = Text;
  increment = GraphLCDSetup.ScrollSpeed;
  position = 0;
  if (GraphLCDSetup.ScrollMode != 0 &&
      font->WidthS(text) > xmax - x + 1)
    active = true;
  else
    active = false;
  update = false;
  lastUpdate = time_ms() + 2000;
}

void cScroller::Draw(cGraphLCDGraphLib * lib)
{
  if (!active)
  {
    lib->Text(x, y, xmax, text, font, true, colorBlack, colorWhite, 0);
  }
  else
  {
    if (update)
    {
      if (increment > 0)
      {
        if (font->WidthS(text) - position + font->Width() * 5 < increment)
        {
          increment = 0;
          position = 0;
        }
      }
      else
      {
        if (GraphLCDSetup.ScrollMode == 2)
        {
          increment = GraphLCDSetup.ScrollSpeed;
        }
        else
        {
          active = false;
        }
      }
      position += increment;
      lastUpdate = time_ms();
      update = false;
    }
    lib->Text(x, y, xmax, text, font, true, colorBlack, colorWhite, position);
    if (font->WidthS(text) - position <= xmax - x + 10 + font->Width() * 5)
      lib->Text(x + font->WidthS(text) - position + font->Width() * 5, y, xmax, text, font, true, colorBlack, colorWhite, 0);
  }
}

